package co.inmobi.plugin.inmobi_plugin

import android.Manifest
import android.annotation.SuppressLint
import android.content.Context
import android.content.SharedPreferences
import android.content.pm.PackageManager
import android.provider.Settings
import android.telephony.TelephonyManager
import android.text.TextUtils
import androidx.annotation.NonNull
import androidx.core.content.ContextCompat
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import java.util.UUID

/** InmobiPlugin */
class InmobiPlugin : FlutterPlugin, MethodCallHandler {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel
    private lateinit var context: Context

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        context = flutterPluginBinding.applicationContext
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "inmobi_plugin")
        channel.setMethodCallHandler(this)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        if (call.method == "getDeviceId") {
            result.success(getUserId(context))
        } else {
            result.notImplemented()
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    private fun getSharedPreferencesSP(): SharedPreferences {
        return this.context.getSharedPreferences("yokara", 0)
    }

    private fun getStringSharedPreference(key: String, defValue: String): String {
        return getSharedPreferencesSP().getString(key, defValue).toString()
    }

    private fun setStringSharedPreference(key: String?, value: String?) {
        getSharedPreferencesSP().edit().putString(key, value).commit()
    }

    private fun setUserIdSP(userId: String?) {
        setStringSharedPreference("userId", userId)
    }

    fun getUserId(context: Context): String? {
        if (getUserIdSP() == null)
            setUserIdSP(getDeviceId(context))
        return getUserIdSP()
    }

    private fun getUserIdSP(): String? {
        val string: String = getStringSharedPreference("userId", "")
        return if (TextUtils.isEmpty(string)) {
            null
        } else string
    }

    @SuppressLint("HardwareIds")
    private fun getDeviceId(context: Context): String? {
        var deviceUuid = UUID.randomUUID()
        if (ContextCompat.checkSelfPermission(
                context,
                Manifest.permission.READ_PHONE_STATE
            )
            != PackageManager.PERMISSION_GRANTED
        ) {
            val androidId: String = Settings.Secure.getString(
                context.contentResolver,
                Settings.Secure.ANDROID_ID
            ).toString()
            deviceUuid = UUID(
                androidId.hashCode().toLong(),
                androidId.hashCode().toLong()
            )
        } else {
            val tm = context
                .getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager
            val tmDevice: String = "" + tm.deviceId
            val tmSerial: String = "" + tm.simSerialNumber
            val androidId: String = (""
                    + Settings.Secure.getString(
                context.contentResolver,
                Settings.Secure.ANDROID_ID
            ))
            deviceUuid = UUID(
                androidId.hashCode().toLong(),
                tmDevice.hashCode().toLong() shl 32 or tmSerial.hashCode().toLong()
            )
        }
        return deviceUuid.toString()
    }
}
